simula_joc = function(canviar = FALSE, portes = 3L, seed = NULL) {
  
  # comprovem que ens han passat els parametres be
  if (!is.logical(canviar) | is.na(canviar)) stop("`canviar` ha de ser TRUE o 
                                                  FALSE")
  if (is.double(portes)) {
    portes = as.integer(portes)
    warning(paste("double introduit per `portes`: passat af", portes))
  }
  if (!is.integer(portes) | portes < 2) stop("`portes` ha de ser un int positiu 
                                             > 2")
  
  # posem la seed
  if (!is.null(seed)) set.seed(seed)
  
  # creem una llista amb les portes
  ll_portes = seq_len(portes)
  
  # creem una llista amb booleans dient quines portes s'obren
  # comencem amb totes les portes tancades
  tancades = rep(TRUE, length = portes)
  
  # escollim un index on estara el cotxe
  cotxe = sample(ll_portes, size = 1)
  
  # porta escollida inicialment pel jugador
  porta_escollida = sample(ll_portes, size = 1)
  
  # El Monty Hall obre totes les portes que el jugador no ha escollit menys una
  # si hem escollit la del cotxe, la que queda tindra una cabra
  # si hem escollit una amb cabra, la que queda tindra el cotxe
  if (cotxe == porta_escollida) {
    # Si hem escollit el cotxe en Monty Hall pot obrir totes les portes excepte
    # dues:
    #   la nostra
    #   i una amb una cabra
    tancades[sample(ll_portes[-cotxe], size = portes - 2)] = FALSE
  } else {
    # sino, en Monty Hall pot obrir totes les portes menys la nostra i la que 
    # te el cotxe
    tancades[-c(cotxe, porta_escollida)] = FALSE
  }
  
  # ens pregunta si volem canviar de porta
  if (canviar) {
    porta_escollida = ll_portes[tancades][ll_portes[tancades] != porta_escollida]
  }
  
  # retornem un boolea segons si hem guanyat o no
  return(porta_escollida == cotxe)
}
#' Simula N jocs de Monty Hall
#' Crida la funcio `simula_joc()` `n` vegades
#' 
#' @param n Enter que ens diu quants cops cridar a `simula_joc()` 
#' @param seed Una seed que ens definira el nombre aleatori. Per defecte es 
#' `NULL`
#'
#' @return Un vector boolea que indica si es guanya o es perd a cada joc
#'
#' @examples 
#' simula_jocs(n = 100)
#' simula_jocs(n = 500, canviar = TRUE)
#' simula_jocs(n = 1000, seed = 123, canviar = TRUE, portes = 5L)
simula_jocs = function(n, seed = NULL, canviar = FALSE, ...) {
  # si ens han passat un double el transformem a int
  if (is.double(n)) {
    n = as.integer(n)
  }
  if (!is.integer(n) | n < 1) stop("`n_games` ha de ser un enter positiu > 1")
  # si ens donen seed, s'introdueix
  if (!is.null(seed)) set.seed(seed)
  # calculem els n jocs
  return(vapply(rep(canviar, n), simula_joc, logical(1), ...))
}
