library("tibble")
library("purrr")
library("ggplot2")
library("tidyr")
library("dplyr")
library("latex2exp")
library("BrailleR")

# funcio que calcula la distribució a posteriori del nombre estimat de tancs
# donades les observacions x
distr_tancs <- function(N, x){
  
  n <- length(x)
  m <- max(x)
  
  # calculem el log del denominador ((m sobre n)^-1 * (m / n - 1))
  logDen <- -lchoose(m, n) + log(m) - log(n-1)
  
  # lchoose calcula ln(. sobre n), en aquest cas, . es N perquè amb map_dbl 
  # estem iterant sobre N, fent que per cadascun es calculi la distribucio
  # if_else(condicio, true, false) ens diu que si N es més petit que el maxim 
  # de les x llavors la sortida haura de ser 0 perquè no pot ser que el nombre
  # de tancs sigui menor que un ja observat
  # es multiplica per 100 per expressar en tant per cent la probabilitat
  map_dbl(N, ~{if_else(. < m, 0, 100*exp(-lchoose(., n) - logDen))})
}
# generem 10 nombres entre 1 i 100
x <- sample(seq(100))[1:10]
x

# llistem els nombres de 80 a 150
N <- seq(80,150)
# construim un data frame d'una taula amb la primera columna la llista N
# i segona columna les probabilitats calculades a dgtp amb N i x
taula <- tibble(N = N, p = distr_tancs(N, x))
View(taula)

# creem el grafic amb ggplot usant punts i segments. labs creen els noms dels
# eixos i ggtitle el titol
grafic <- ggplot(taula, aes(label=p)) +
geom_point(aes(x = N, y = p))  +
geom_segment(aes(x = N, xend = N, y = 0, yend = p)) +
labs(x = TeX('$N$'),
     y = TeX('Probabilidad de N condicionada por x de 1 a n en %')) +
  ggtitle("Estimación de tanques")

# usem BrailleR per descriure els punts del grafic amb la threshold que es els
# punts que volem descriure
VI(grafic, describe=TRUE, threshold = 100)


# calculs addicionals que poden ser interessants:

# mediana del nombre de tancs estimat
gtp_mediana <- function(x){
  
  m <- max(x)
  
  s <- distr_tancs(m, x)
  i <- m
  while (s + distr_tancs(i + 1, x) < 0.5) {
    i <- i + 1
    s <- s + distr_tancs(i, x)
  }
  
  i
}
gtp_mediana(x)

# mitjana del nombre de tancs
gtp_mitjana <- function(x){
  m <- max(x)
  n <- length(x)
  
  (n-1)/(n-2) * (m - 1)
}
gtp_mitjana(x)

# interval de mes alta densitat a posteriori
gtp_mdp <- function(x, p = 0.95){
  m <- max(x)
  
  s <- distr_tancs(m, x)
  i <- m
  while (s + distr_tancs(i + 1, x) < p) {
    i <- i + 1
    s <- s + distr_tancs(i, x)
  }
  
  c(m, i)
}
gtp_mdp(x)

